@@ -1034,7 +1034,8 @@ class RecordingHandle:
                  monitoring_intervals,
                  do_profile,
                  kdamonds, add_child_tasks, record_mem_footprint,
-                 record_vmas, record_proc_stats, timeout):
+                 record_vmas, record_proc_stats, timeout, snapshot_request,
+                 snapshot_interval_sec, snapshot_count):
         self.tracepoint = tracepoint
         self.file_path = file_path
         self.file_format = file_format
@@ -1055,6 +1056,10 @@ class RecordingHandle:
 
         self.timeout = timeout
 
+        self.snapshot_request = snapshot_request
+        self.snapshot_interval_sec = snapshot_interval_sec
+        self.snapshot_count = snapshot_count
+
     def will_take_awhile(self):
         if self.snapshot_interval_sec == 0 and self.snapshot_count < 5:
             return False
@@ -1158,6 +1163,21 @@ def finish_recording(handle):
 
 # for snapshot
 
+def current_kdamonds_interval_updated():
+    kdamonds = _damon.current_kdamonds()
+    intervals_autotuning = False
+    for kd in kdamonds:
+        for ctx in kd.contexts:
+            if ctx.intervals.intervals_goal.enabled():
+                intervals_autotuning = True
+                break
+        if intervals_autotuning:
+            break
+    if not intervals_autotuning:
+        return kdamonds
+    _damon.update_tuned_intervals()
+    return _damon.current_kdamonds()
+
 def find_install_scheme(scheme_to_find):
     '''Install given scheme to all contexts if effectively same scheme is not
     installed.
@@ -1166,7 +1186,7 @@ def find_install_scheme(scheme_to_find):
     '''
     installed = False
     indices = []
-    kdamonds = _damon.current_kdamonds()
+    kdamonds = current_kdamonds_interval_updated()
     for kidx, kdamond in enumerate(kdamonds):
         for cidx, ctx in enumerate(kdamond.contexts):
             ctx_has_the_scheme = False
@@ -1188,9 +1208,9 @@ def find_install_scheme(scheme_to_find):
     if installed:
         err = _damon.commit(kdamonds)
         if err is not None:
-            return (False, [],
+            return (False, [], None,
                     'committing scheme installed kdamonds failed: %s' % err)
-    return installed, indices, None
+    return installed, indices, kdamonds, None
 
 def can_merge(left_region, right_region):
     if left_region.sz_filter_passed or right_region.sz_filter_passed:
@@ -1344,6 +1364,17 @@ def update_get_snapshot_records(kdamond_idxs, scheme_idxs,
     records = tried_regions_to_records_of(scheme_idxs, merge_regions)
     return records, None
 
+def kdamonds_copy_intervals(src_kdamonds, dst_kdamonds):
+    for kidx, kdamond in enumerate(src_kdamonds):
+        if kidx >= len(dst_kdamonds):
+            continue
+        dst_kdamond = dst_kdamonds[kidx]
+        for cidx, ctx in enumerate(kdamond.contexts):
+            if cidx >= len(dst_kdamond.contexts):
+                continue
+            dst_ctx = dst_kdamond.contexts[cidx]
+            dst_ctx.intervals = ctx.intervals
+
 def get_snapshot_records(monitor_scheme, total_sz_only, merge_regions):
     'return DamonRecord objects each having single DamonSnapshot and an error'
     running_kdamond_idxs = _damon.running_kdamond_idxs()
@@ -1356,7 +1387,8 @@ def get_snapshot_records(monitor_scheme, total_sz_only, merge_regions):
     if err is not None:
         return None, 'vaddr region install failed (%s)' % err
 
-    installed, idxs, err = find_install_scheme(monitor_scheme)
+    installed, idxs, updated_kdamonds, err = find_install_scheme(
+            monitor_scheme)
     if err:
         return None, 'monitoring scheme install failed: %s' % err
 
@@ -1364,6 +1396,7 @@ def get_snapshot_records(monitor_scheme, total_sz_only, merge_regions):
             total_sz_only, merge_regions)
 
     if installed:
+        kdamonds_copy_intervals(updated_kdamonds, orig_kdamonds)
         uninstall_err = _damon.commit(orig_kdamonds)
         if uninstall_err:
             errmsg = 'monitoring scheme uninstall failed: %s' % uninstall_err
