@@ -127,6 +127,15 @@ snapshot_formatters = [
                   recency_hist_str(snapshot, record, fmt, True),
                   ' '.join(['last accessed time to total size of',
                             'DAMOS filters (df) passed regions histogram'])),
+        Formatter('<recency percentiles>',
+                  lambda snapshot, record, fmt:
+                  recency_percentiles(snapshot, record, fmt, False),
+                  'per-byte last accessed time distribution in percentiles'),
+        Formatter('<df-passed recency percentiles>',
+                  lambda snapshot, record, fmt:
+                  recency_percentiles(snapshot, record, fmt, True),
+                  ' '.join(['per-df-passed byte last accessed time',
+                            'distribution in percentiles'])),
         Formatter('<heatmap>',
                   lambda snapshot, record, fmt:
                   heatmap_str(snapshot, record, fmt),
@@ -373,7 +382,8 @@ def get_unsorted_histogram(snapshot, fmt, get_x_fn, get_y_fn):
         hist[xval] += get_y_fn(region, fmt)
     return hist
 
-def get_sorted_ranged_historgram(hist, fmt, fmt_x_fn, parse_x_fn, fmt_y_fn):
+def get_sorted_ranged_historgram(
+        hist, fmt, fmt_x_fn, parse_x_fn, fmt_y_fn, y_aggr_fn):
     metrics = sorted(hist.keys())
     hist2 = []
     if fmt.hist_ranges is not None:
@@ -389,7 +399,8 @@ def get_sorted_ranged_historgram(hist, fmt, fmt_x_fn, parse_x_fn, fmt_y_fn):
                 fmt.hist_cumulate)
     raw = fmt.raw_number
     for min_m, max_m in hist_ranges:
-        yval = sum([hist[x] for x in metrics if x >= min_m and x < max_m])
+        yval = y_aggr_fn(
+                [hist[x] for x in metrics if x >= min_m and x < max_m])
         metric_range_str = '[%s, %s)' % (
                 fmt_x_fn(min_m, raw), fmt_x_fn(max_m, raw))
         yval_str = fmt_y_fn(yval, raw)
@@ -399,7 +410,7 @@ def get_sorted_ranged_historgram(hist, fmt, fmt_x_fn, parse_x_fn, fmt_y_fn):
 def get_sz_region(region, fmt):
     return region.size()
 
-def get_df_passed_sz_region(region):
+def get_df_passed_sz_region(region, fmt):
     return region.sz_filter_passed
 
 def sz_hist_str(snapshot, fmt, df_passed_sz, get_metric_fn, fmt_metric_fn,
@@ -412,7 +423,8 @@ def sz_hist_str(snapshot, fmt, df_passed_sz, get_metric_fn, fmt_metric_fn,
         get_y_fn = get_sz_region
     hist = get_unsorted_histogram(snapshot, fmt, get_metric_fn, get_y_fn)
     hist2 = get_sorted_ranged_historgram(
-            hist, fmt, fmt_metric_fn, parse_metric_fn, _damo_fmt_str.format_sz)
+            hist, fmt, fmt_metric_fn, parse_metric_fn, _damo_fmt_str.format_sz,
+            lambda sz_list: sum(sz_list))
 
     return histogram_str(hist2)
 
@@ -431,25 +443,71 @@ def get_last_used_time(region, fmt):
     return region.age.usec
 
 def recency_hist_str(snapshot, record, fmt, df_passed_sz):
-    return sz_hist_str(snapshot, fmt, df_passed_sz, get_last_used_time,
-                       _damo_fmt_str.format_time_us, _damo_fmt_str.text_to_us)
+    if len(snapshot.regions) == 0:
+        return 'no region in snapshot'
 
-class HistFns:
-    get = None
-    fmt = None
-    parse = None
+    get_metric_fn = get_last_used_time
+    fmt_metric_fn = _damo_fmt_str.format_time_us
+    parse_metric_fn = _damo_fmt_str.text_to_us
 
-    def __init__(self, get, fmt, parse):
-        self.get = get
-        self.fmt = fmt
-        self.parse = parse
+    if df_passed_sz is True:
+        get_y_fn = get_df_passed_sz_region
+    else:
+        get_y_fn = get_sz_region
+    y_fmt_fn = _damo_fmt_str.format_sz
+    y_aggr_fn = lambda vals: sum(vals)
 
-hist_fns_map = {
-        'recency': HistFns(get_last_used_time, _damo_fmt_str.format_time_us,
-                           _damo_fmt_str.text_to_us),
-        'sz': HistFns(get_sz_region, _damo_fmt_str.format_sz,
-                      _damo_fmt_str.text_to_bytes),
-        }
+    hist = get_unsorted_histogram(snapshot, fmt, get_metric_fn, get_y_fn)
+    hist2 = get_sorted_ranged_historgram(
+            hist, fmt, fmt_metric_fn, parse_metric_fn, y_fmt_fn, y_aggr_fn)
+
+    return histogram_str(hist2)
+
+def recency_percentiles(snapshot, record, fmt, df_passed):
+    if len(snapshot.regions) == 0:
+        return 'no region in snapshot'
+    regions = sorted(snapshot.regions,
+                     key=lambda r: get_last_used_time(r, fmt))
+    if df_passed is True:
+        total_sz = sum(r.sz_filter_passed for r in regions)
+    else:
+        total_sz = sum(r.size() for r in regions)
+    percentiles_to_show = [0, 1, 25, 50, 75, 99]
+    percentile = 0
+    percentile_values = []
+    if df_passed is True:
+        lines = ['<df-passed percentile> <last accessed time>']
+    else:
+        lines = ['<percentile> <last accessed time>']
+    for r in regions:
+        if df_passed is True:
+            percentile += r.sz_filter_passed * 100 / total_sz
+        else:
+            percentile += r.size() * 100 / total_sz
+        while len(percentiles_to_show) > 0:
+            if percentile < percentiles_to_show[0]:
+                break
+            percentile_values.append(
+                    [percentiles_to_show[0], get_last_used_time(r, fmt)])
+            percentiles_to_show = percentiles_to_show[1:]
+        if percentile >= 100.0:
+            break
+    percentile_values.append([100, get_last_used_time(r, fmt)])
+
+    min_val = percentile_values[0][-1]
+    max_val = percentile_values[-1][-1]
+    max_dots = 20
+    if max_val != min_val:
+        val_per_dot = (max_val - min_val) / max_dots
+    else:
+        val_per_dot = 1
+    for percentile, val in percentile_values:
+        bar_length = int((val - min_val) / val_per_dot)
+        bar = '|%s%s|' % ('*' * bar_length, ' ' * (max_dots - bar_length))
+        lines.append(
+                '%3d %18s %s' %
+                (percentile, _damo_fmt_str.format_time_us(val, fmt.raw), bar))
+    return '\n'.join(lines)
 
 def temperature_str(region, raw, fmt):
     temperature = temperature_of(region, fmt.temperature_weights)
@@ -1165,6 +1223,38 @@ def set_formats_hist_style(args, fmt, records):
 
 def set_formats(args, records):
     fmt = ReportFormat.from_args(args)
+
+    if args.format is not None:
+        if len(args.format) == 1 and len(args.format[0]) == 1:
+            fmt_string = args.format[0][0]
+            if os.path.isfile(fmt_string):
+                with open(fmt_string, 'r') as f:
+                    fmt_string = f.read()
+            try:
+                return ReportFormat.from_kvpairs(json.loads(fmt_string)), None
+            except:
+                return None, 'wrong --format option'
+        for format_fields in args.format:
+            if len(format_fields) != 3:
+                return None, 'wrong --format option (%s)' % format_fields
+            action, target_area, fmt_string = format_fields
+            if not action in ['set', 'append']:
+                return None, 'wrong --format action (%s)' % format_fields
+            if not target_area in ['record_head', 'snapshot_head', 'region',
+                                   'snapshot_tail', 'record_tail']:
+                return None, 'wrong --format target area (%s)' % format_fields
+            if action == 'set':
+                if target_area == 'record_head':
+                    fmt.format_record_head = fmt_string
+                elif target_area == 'snapshot_head':
+                    fmt.format_snapshot_head = fmt_string
+                elif target_area == 'region':
+                    fmt.format_region = fmt_string
+                elif target_area == 'snapshot_tail':
+                    fmt.format_snapshot_tail = fmt_string
+                elif target_area == 'record_tail':
+                    fmt.format_record_tail = fmt_string
+
     if args.style == 'simple-boxes':
         fmt.format_snapshot_head = default_snapshot_head_format_without_heatmap
         fmt.format_region = '<box> size <size> access rate <access rate> age <age>'
@@ -1207,7 +1297,7 @@ def set_formats(args, records):
             fmt.format_snapshot_tail = ('%s\n<region box description>' %
                     fmt.format_record_tail)
     if len(records) == 0:
-        return fmt
+        return fmt, None
 
     if fmt.format_record_head == None:
         if len(records) > 1:
@@ -1221,21 +1311,20 @@ def set_formats(args, records):
                 fmt.format_region += ' df-passed <filters passed bytes>'
                 break
 
+    ops_filters_installed = False
+    intervals_tuning_enabled = False
+    for record in records:
+        if len(record.scheme_filters) > 0:
+            ops_filters_installed = True
+        if record.intervals is not None:
+            if record.intervals.intervals_goal.enabled():
+                intervals_tuning_enabled = True
+        if ops_filters_installed and intervals_tuning_enabled:
+            break
+
     if fmt.format_snapshot_tail == default_snapshot_tail_format:
-        ops_filters_installed = False
-        intervals_tuning_enabled = False
-        for record in records:
-            if len(record.scheme_filters) > 0:
-                ops_filters_installed = True
-            if record.intervals is not None:
-                if record.intervals.intervals_goal.enabled():
-                    intervals_tuning_enabled = True
-            if ops_filters_installed and intervals_tuning_enabled:
-                break
         if ops_filters_installed:
             fmt.format_snapshot_tail = default_snapshot_tail_format_filter_installed
-        if intervals_tuning_enabled:
-            fmt.format_snapshot_tail += '\nintervals tuning status: <intervals tuning status>'
         # further check if scheme action is not stat
         if args.tried_regions_of is not None:
             fmt.format_snapshot_tail += '\nscheme stats\n<damos stats>'
@@ -1254,7 +1343,24 @@ def set_formats(args, records):
         fmt.format_snapshot_head += '\n# damos filters (df): <filters passed type>'
         fmt.format_snapshot_head += '\ndf-pass: <filters passed heatmap>'
 
-    return fmt
+    if fmt.format_record_tail == '' and intervals_tuning_enabled:
+        fmt.format_record_tail = 'monitoring intervals: <intervals>'
+
+    if args.format is not None:
+        for action, target_area, fmt_string in args.format:
+            if action == 'append':
+                if target_area == 'record_head':
+                    fmt.format_record_head += fmt_string
+                elif target_area == 'snapshot_head':
+                    fmt.format_snapshot_head += fmt_string
+                elif target_area == 'region':
+                    fmt.format_region += fmt_string
+                elif target_area == 'snapshot_tail':
+                    fmt.format_snapshot_tail += fmt_string
+                elif target_area == 'record_tail':
+                    fmt.format_record_tail += fmt_string
+
+    return fmt, None
 
 def handle_ls_keywords(args):
     if args.ls_record_format_keywords:
@@ -1418,14 +1524,9 @@ def read_and_show(args):
                 print('some records lack the intervals information')
                 exit(1)
 
-        if args.format is not None:
-            fmt_string = args.format
-            if os.path.isfile(fmt_string):
-                with open(fmt_string, 'r') as f:
-                    fmt_string = f.read()
-            fmt = ReportFormat.from_kvpairs(json.loads(fmt_string))
-        else:
-            fmt = set_formats(args, records)
+        fmt, err = set_formats(args, records)
+        if err is not None:
+            print('format setting failed (%s)' % err)
 
         if args.on_cache is not None:
             sz_cache = _damo_fmt_str.text_to_bytes(args.on_cache[0])
@@ -1488,29 +1589,39 @@ def add_fmt_args(parser, hide_help=False):
     parser.add_argument('--hist_cumulate', action='store_true',
                         help='draw histogram in cumulative way')
 
+    fmt_flexible_metavar='<set|append> <record_head|snapshot_head|region|snapshot_tail|record_tail> <format string>'
+
+    # This option also receives json string or file.  But, no real use case is
+    # found.  Keep supporting for now, but hide from the help message.
+    parser.add_argument('--format', metavar=fmt_flexible_metavar,
+                        action='append', nargs='+',
+                        help='visualization format in json format')
+
     # don't set default for record head and snapshot head because it depends on
     # given number of record and snapshots.  Decide those in set_formats().
+    # below five options can be replaced with --format.  Keep backward
+    # compatibility support, but hide from help messages.
     parser.add_argument(
             '--format_record_head', metavar='<template>',
-            help='output format to show at the beginning of each record'
-            if not hide_help else argparse.SUPPRESS)
+            # help='output format to show at the beginning of each record'
+            help=argparse.SUPPRESS)
     parser.add_argument('--format_record_tail', metavar='<template>',
                         default='',
-                        help='output format to show at the end of each record'
-                        if not hide_help else argparse.SUPPRESS)
+                        # help='output format to show at the end of each record'
+                        help=argparse.SUPPRESS)
     parser.add_argument(
             '--format_snapshot_head', metavar='<template>',
-            help='output format to show at the beginning of each snapshot'
-            if not hide_help else argparse.SUPPRESS)
+            # help='output format to show at the beginning of each snapshot'
+            help=argparse.SUPPRESS)
     parser.add_argument(
             '--format_snapshot_tail', metavar='<template>',
             default=default_snapshot_tail_format,
-            help='output format to show at the end of each snapshot'
-            if not hide_help else argparse.SUPPRESS)
+            # help='output format to show at the end of each snapshot'
+            help=argparse.SUPPRESS)
     parser.add_argument('--format_region', metavar='<template>',
                         default=default_region_format,
-                        help='output format to show for each memory region'
-                        if not hide_help else argparse.SUPPRESS)
+                        # help='output format to show for each memory region'
+                        help=argparse.SUPPRESS)
 
     # for snapshot heatmap
     parser.add_argument(
@@ -1627,8 +1738,7 @@ def set_argparser(parser):
             help='show tried regions of given schemes')
     _damo_records.set_snapshot_damos_filters_option(parser)
     add_fmt_args(parser, hide_help=True)
-    parser.add_argument('--format', metavar='<json string>',
-                        help='visualization format in json format')
+
     parser.add_argument('--exec', metavar='<command or \'interpreter\'>',
                         help='execute python code with the records')
     parser.add_argument(
