@@ -24,6 +24,11 @@ test_report() {
 
 mkdir -p results
 
+damo_report_style="$damo report access --input damon.data.snapshot --style"
+test_report "$damo_report_style recency-sz-hist" "recency_sz_hist"
+
+test_report "$damo_report_style temperature-sz-hist" "temperature_sz_hist"
+
 damo_report_raw="$damo report access --raw_form --input"
 
 test_report "$damo_report_raw damon.data" "raw"
